<template>
  <div
    :class="{ 'border-red-400': errors }"
    class="group relative flex h-48 items-center justify-center rounded-md border border-dashed border-gray-900/25 hover:cursor-pointer hover:bg-gray-50"
    @click="inputElement?.click()"
    @dragenter.prevent="dropZoneEnabled = true"
    @dragleave.prevent="dropZoneEnabled = false"
    @dragover.prevent="dropZoneEnabled = true"
    @drop.prevent="drop"
  >
    <div
      v-if="previewImage"
      class="flex h-full w-full p-4"
    >
      <img
        :src="previewImage"
        alt="Preview"
        class="m-auto h-full"
      />
    </div>
    <div
      :class="{ 'absolute hidden rounded-md bg-gray-50/80 group-hover:flex': previewImage }"
      class="flex h-full w-full flex-col items-center px-6 py-10"
    >
      <ImageSolidIcon
        :class="{ 'fill-gray-500': previewImage }"
        class="mx-auto h-12 w-12 fill-gray-300"
      />
      <div class="mt-4 flex text-sm leading-6 text-gray-600">
        <button
          class="relative rounded font-semibold text-primary-500 hover:cursor-pointer hover:text-primary-500 focus:outline-primary-500"
          type="button"
        >
          {{ $t("app.form.image.upload") }}
        </button>
        <p class="pl-1">
          {{ $t("app.form.image.drag") }}
        </p>
      </div>
      <p class="text-xs leading-5 text-gray-600">
        {{ $t("app.form.image.limits") }}
      </p>
    </div>
    <input
      ref="inputElement"
      accept="image/*"
      class="sr-only"
      name="file-upload"
      type="file"
      @input="input"
    />
  </div>
</template>

<script lang="ts" setup>
import ImageSolidIcon from "@/svg/icons/image-solid.svg?component";
import type { ComputedRef } from "vue";

const props = defineProps<{
  modelValue: string | File;
}>();

const emit = defineEmits<{
  "update:modelValue": [value: File];
}>();

const errors = inject<ComputedRef<boolean>>("errors");

watch(
  () => props.modelValue,
  () => {
    if (typeof props.modelValue === "string") {
      previewImage.value = props.modelValue.toString() ?? undefined;
    }
  },
);

const previewImage = ref<string>(props.modelValue.toString() ?? undefined);
const dropZoneEnabled = ref(false);
const inputElement = ref<HTMLInputElement>();

const input = ($event: Event) => {
  dropZoneEnabled.value = false;
  const files = ($event.target as HTMLInputElement & EventTarget)?.files ?? [];
  if (files.length > 0) {
    select(files[0]);
  }
};
const drop = ($event: DragEvent) => {
  dropZoneEnabled.value = false;
  const files = ($event.dataTransfer as DragEvent & DataTransfer)?.files ?? [];
  if (files.length > 0) {
    select(files[0]);
  }
};

const select = (file: File) => {
  if (new RegExp("(^image)(\\/)[a-zA-Z0-9_]*").test(file.type)) {
    previewImage.value = URL.createObjectURL(file);
    emit("update:modelValue", file);
  }
};
</script>
